@@ -86,15 +86,6 @@ def value_fill_max(x, dic):
     def test_parallel_enumerate(self):
         self.run_test('def parallel_enumerate(l):\n k = [0]*(len(l) + 1)\n "omp parallel for"\n for i,j in enumerate(l):\n  k[i+1] = j\n return k', list(range(1000)), parallel_enumerate=[List[int]])
 
-    def test_ultra_nested_functions(self):
-        code = '''
-def ultra_nested_function(n):
-	def foo(y):
-		def bar(t): return t
-		return bar(y)
-	return foo(n)'''
-        self.run_test(code, 42, ultra_nested_function=[int])
-
     def test_generator_sum(self):
         code = '''
 def generator_sum(l0,l1):
