@@ -9,7 +9,6 @@ from pythran.typing import NDArray, List, Tuple
 np_version = version.parse(numpy.version.version)
 
 
-@TestEnv.module
 class TestNumpyFunc2(TestEnv):
 
     def test_nonzero0(self):
@@ -178,7 +177,7 @@ class TestNumpyFunc2(TestEnv):
                   np_correlate_8=[NDArray[numpy.complex64,:],NDArray[numpy.complex64,:]])
 
     def test_correlate_9(self):
-        dtype = numpy.float
+        dtype = float
         self.run_test("def np_correlate_9(a,b):\n from numpy import correlate\n return correlate(a,b,'full')",
                   numpy.arange(7).astype(dtype)  + 1j*numpy.arange(7).astype(dtype),
                   numpy.arange(12).astype(dtype) + 1j*numpy.arange(12).astype(dtype),
@@ -207,6 +206,12 @@ class TestNumpyFunc2(TestEnv):
                   numpy.arange(10,dtype=float),
                   np_convolve_2=[NDArray[float,:],NDArray[float,:]])
 
+    def test_convolve_2b(self):
+        self.run_test("def np_convolve_2b(a,b):\n from numpy import convolve\n return convolve(a,b)",
+                  numpy.arange(12,dtype=numpy.float128),
+                  numpy.arange(10,dtype=numpy.float128),
+                  np_convolve_2b=[NDArray[numpy.float128,:],NDArray[numpy.float128,:]])
+
     def test_convolve_3(self):
         self.run_test("def np_convolve_3(a,b):\n from numpy import convolve\n return convolve(a,b,'valid')",
                   numpy.arange(12,dtype=float),
@@ -246,7 +251,7 @@ class TestNumpyFunc2(TestEnv):
                   np_convolve_8=[NDArray[numpy.complex64,:],NDArray[numpy.complex64,:]])
 
     def test_convolve_9(self):
-        dtype = numpy.float
+        dtype = float
         self.run_test("def np_convolve_9(a,b):\n from numpy import convolve\n return convolve(a,b,'full')",
                   numpy.arange(7).astype(dtype) + 1j*  numpy.arange(7).astype(dtype),
                   numpy.arange(12).astype(dtype) + 1j* numpy.arange(12).astype(dtype),
@@ -304,6 +309,12 @@ def test_copy0(x):
     def test_clip1(self):
         self.run_test("def np_clip1(a): from numpy import  clip ; return clip(a,3,6)", numpy.arange(10), np_clip1=[NDArray[int,:]])
 
+    def test_clip2(self):
+        self.run_test("def np_clip2(a): from numpy import  clip ; return clip(a,3,None)", numpy.arange(10), np_clip2=[NDArray[int,:]])
+
+    def test_clip3(self):
+        self.run_test("def np_clip3(a): from numpy import  clip ; return clip(a,None, 6)", numpy.arange(10), np_clip3=[NDArray[int,:]])
+
     def test_concatenate0(self):
         self.run_test("def np_concatenate0(a): from numpy import array, concatenate ; b = array([[5, 6]]) ; return concatenate((a,b))", numpy.array([[1, 2], [3, 4]]), np_concatenate0=[NDArray[int,:,:]])
 
@@ -505,11 +516,12 @@ def test_copy0(x):
     def test_asarray6(self):
         self.run_test("def np_asarray6(a):\n from numpy import asarray\n return asarray(a, dtype=int)", 1.5, np_asarray6=[float])
 
-    def test_asfarray0(self):
-        self.run_test("def np_asfarray0(a):\n from numpy import asfarray; b = asfarray(a) ; return a is b", numpy.arange(3.), np_asfarray0=[NDArray[float,:]])
+    if hasattr(numpy, 'farray'):
+        def test_asfarray0(self):
+            self.run_test("def np_asfarray0(a):\n from numpy import asfarray; b = asfarray(a) ; return a is b", numpy.arange(3.), np_asfarray0=[NDArray[float,:]])
 
-    def test_asfarray1(self):
-        self.run_test("def np_asfarray1(a):\n from numpy import asfarray; b = asfarray(a) ; return a is not b", numpy.arange(3), np_asfarray1=[NDArray[int,:]])
+        def test_asfarray1(self):
+            self.run_test("def np_asfarray1(a):\n from numpy import asfarray; b = asfarray(a) ; return a is not b", numpy.arange(3), np_asfarray1=[NDArray[int,:]])
 
     def test_astype0(self):
         self.run_test("def np_astype0(a):\n return a.astype(float)", numpy.arange(3), np_astype0=[NDArray[int,:]])
@@ -532,6 +544,21 @@ def test_copy0(x):
     def test_array_split3(self):
         self.run_test("def np_array_split3(a): from numpy import array_split, array2string ; return list(map(array2string,array_split(a, [0,1,5])))", numpy.arange(12).reshape(6,2), np_array_split3=[NDArray[int,:,:]])
 
+    def test_array_split4(self):
+        self.run_test("def np_array_split4(a): from numpy import array_split, array2string ; return list(map(array2string,array_split(a + 1, 3)))", numpy.arange(12).reshape(6,2), np_array_split4=[NDArray[int,:,:]])
+
+    def test_array_split5(self):
+        self.run_test("""
+            def np_array_split5(a):
+                from numpy import array_split, array2string
+                split = array_split(a, 3)
+                split[0][:] = 1
+                return list(map(array2string, split)), a""",
+            numpy.arange(12).reshape(6,2), np_array_split5=[NDArray[int,:,:]])
+
+    def test_array_split6(self):
+        self.run_test("def np_array_split6(a): from numpy import array_split, array2string ; return list(map(array2string,array_split(a[1:-1], 3)))", numpy.arange(22).reshape(11,2), np_array_split6=[NDArray[int,:,:]])
+
     def test_array_equiv0(self):
         self.run_test("def np_array_equiv0(a): from numpy import array_equiv ;  b = [1,2] ; return array_equiv(a,b)", [1, 2], np_array_equiv0=[List[int]])
 
@@ -599,6 +626,21 @@ def test_copy0(x):
     def test_argsort1(self):
         self.run_test("def np_argsort1(x): return x.argsort()", numpy.array([[3, 1, 2], [1 , 2, 3]]), np_argsort1=[NDArray[int,:,:]])
 
+    def test_argsort2(self):
+        self.run_test("def np_argsort2(x): return x.argsort(axis=0)", numpy.array([[3, 1, 2], [1 , 2, 3]]), np_argsort2=[NDArray[int,:,:]])
+
+    def test_argsort3(self):
+        self.run_test("def np_argsort3(x): return x.argsort(axis=1)", numpy.array([[3, 1, 2], [1 , 2, 3]]), np_argsort3=[NDArray[int,:,:]])
+
+    def test_argsort4(self):
+        self.run_test("def np_argsort4(x): return x.argsort(axis=None)", numpy.array([[3, 1, 2], [1 , 2, 3]]), np_argsort4=[NDArray[int,:,:]])
+
+    def test_argsort5(self):
+        self.run_test("def np_argsort5(x): return x.argsort(axis=None,kind=None)", numpy.array([[3, 1, 2], [1 , 2, 3]]), np_argsort5=[NDArray[int,:,:]])
+
+    def test_argsort6(self):
+        self.run_test("def np_argsort6(x): return x.argsort(kind='stable')", numpy.array([[3, 1, 2], [1 , 2, 3]]), np_argsort6=[NDArray[int,:,:]])
+
     def test_argmax0(self):
         self.run_test("def np_argmax0(a): return a.argmax()", numpy.arange(6).reshape(2,3), np_argmax0=[NDArray[int,:,:]])
 
@@ -698,6 +740,12 @@ def test_copy0(x):
     def test_array2D_(self):
         self.run_test("def np_array2D_(a):\n from numpy import array\n return array(a)", [[1,2],[3,4]], np_array2D_=[List[List[int]]])
 
+    def test_array_iter(self):
+        self.run_test("def np_array_iter():\n from numpy import array\n return array(list(reversed(range(0, 300)))) + 3", np_array_iter=[])
+
+    def test_array_tuple(self):
+        self.run_test("def np_array_tuple(x):\n from numpy import array\n return array((1, 2, x))", 3.5, np_array_tuple=[float])
+
     def test_array_typed(self):
         self.run_test("def np_array_typed(a):\n from numpy import array, int64\n return array(a, int64)", [1.,2.,3.], np_array_typed=[List[float]])
 
@@ -752,18 +800,27 @@ def test_copy0(x):
     def test_empty_integral_shape(self):
         self.run_test("def np_empty_integral_shape(n):\n from numpy import empty, uint8\n a = empty(5)\n return a.strides, len(a), n", 3, np_empty_integral_shape=[int])
 
+    def test_empty_empty_shape(self):
+        self.run_test("def np_empty_empty_shape(n):\n from numpy import empty, uint8, shape, ndim\n a = empty(())\n return a.strides, n, shape(a), ndim(a)", 3, np_empty_empty_shape=[int])
+
     def test_ones_uint_shape(self):
         self.run_test("def np_ones_uint_shape(a):\n from numpy import ones, uint32\n a = ones((uint32(a), uint32(a)))\n return a.strides, len(a)", 3, np_ones_uint_shape=[int])
 
     def test_ones_integral_shape(self):
         self.run_test("def np_ones_integral_shape(n):\n from numpy import ones, uint8\n a = ones(5)\n return a.strides, len(a), n", 3, np_ones_integral_shape=[int])
 
+    def test_ones_empty_shape(self):
+        self.run_test("def np_ones_empty_shape(n):\n from numpy import ones, uint8, shape, ndim\n a = ones(())\n return a.strides, n, shape(a), ndim(a)", 3, np_ones_empty_shape=[int])
+
     def test_zeros_uint_shape(self):
         self.run_test("def np_zeros_uint_shape(a):\n from numpy import zeros, int32\n a = zeros((int32(a), int32(a)))\n return a.strides, len(a)", 3, np_zeros_uint_shape=[int])
 
     def test_zeros_integral_shape(self):
         self.run_test("def np_zeros_integral_shape(n):\n from numpy import zeros, uint8\n a = zeros(5)\n return a.strides, len(a), n", 3, np_zeros_integral_shape=[int])
 
+    def test_zeros_empty_shape(self):
+        self.run_test("def np_zeros_empty_shape(n):\n from numpy import zeros, uint8, shape, ndim\n a = zeros(())\n return a.strides, n, shape(a), ndim(a), a.shape, a.ndim", 3, np_zeros_empty_shape=[int])
+
     def test_empty_kwargs(self):
         self.run_test("def np_empty_kwargs(a):\n from numpy import empty\n a = empty(a, dtype=int)\n return a.strides, len(a)", (3, 2), np_empty_kwargs=[Tuple[int, int]])
 
